from django.shortcuts import render, HttpResponseRedirect, reverse, HttpResponse
from django.http import JsonResponse, FileResponse
from django.views.generic import ListView, CreateView, UpdateView
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator

from .models import Spider, Job
from .service import Service
from .periodic_task_creator import make_periodic_task, update_schedule
from .forms import CreateJobForm

BAD_QUERY = 'Bab query'


@method_decorator(login_required, name='post')
class JobListView(ListView):
    template_name = 'list_jobs.html'
    model = Job

    def __init__(self):
        self.service = Service()
        super(JobListView, self).__init__()

    def get(self, request, *args, **kwargs):
        action = request.GET.get('action')
        job_id = request.GET.get('id')
        if action:
            log = self.service.get_log(job_id)
            return HttpResponse(log, content_type='text/plain')
        self.service.update_jobs_status()
        j = Job.objects.all().order_by('-create_time')
        jobs = {
            'deleted': j.deleted(),
            'running': j.running(),
            'finished': j.finished(),
            'pending': j.pending()
        }
        return render(request, self.template_name, {'jobs': jobs})

    def post(self, request, *args, **kwargs):
        if request.is_ajax():
            action = request.POST.get('action')
            job_id = request.POST.get('id')
            return self._logic(action, job_id)
        return HttpResponseRedirect(reverse('job_list'))

    def _logic(self, action, job_id):
        if action == 'run':
            return JsonResponse(self.service.run_job(job_id))
        if action == 'cancel':
            return JsonResponse(self.service.cancel_job(job_id))
        if action == 'update':
            self.service.update_jobs_status()
        if action == 'delete':
            return JsonResponse(self.service.delete_job(job_id))
        if action == 'activate':
            return JsonResponse(self.service.activate_job(job_id))
        if action == 'new':
            return HttpResponseRedirect(reverse('new_job'))
        if action == 'restart':
            return JsonResponse(self.service.restart_job(job_id))
        if action == 'log':
            log = self.service.get_log(job_id)
            return FileResponse(log)


@method_decorator(login_required, name='dispatch')
class NewJobCreateView(CreateView):
    template_name = 'new_job.html'

    def get(self, request, *args, **kwargs):

        spiders = Spider.objects.all()
        return render(request, self.template_name, {'spiders': spiders})

    def post(self, request, *args, **kwargs):
        form = CreateJobForm(request.POST)
        if form.is_valid():
            job = form.save()
        else:
            return HttpResponse('Bad request', status=400)
        interval = request.POST.get('interval', '').upper()
        if interval:
            job.interval = interval
            job.save()
            job.schedule_job = make_periodic_task(job.id, schedule(interval, job.count))
        job.save()

        return HttpResponseRedirect(reverse('job_list'))


@method_decorator(login_required, name='dispatch')
class EditJobView(UpdateView):
    template_name = 'edit_job.html'

    def get(self, request, *args, **kwargs):
        job = Job.objects.get(pk=kwargs.get('id'))
        spiders = Spider.objects.all().exclude(sp_name=job.spider.sp_name)
        return render(request, self.template_name, {'job': job, 'spiders': spiders})

    def post(self, request, *args, **kwargs):
        name = request.POST.get('name')
        spider = Spider.objects.get(pk=request.POST.get('spider'))
        query = validate_query(request.POST.get('query'))
        if query == BAD_QUERY:
            return HttpResponse('Bad request', status=400)
        scrap_old_news = True if request.POST.get('scrap_old_news') else False
        interval = request.POST.get('interval', '').upper()
        count = request.POST.get('count', 1)
        count = 1 if count == '' else int(count)
        job = Job.objects.get(pk=request.POST.get('job'))

        job.name = name
        job.spider = spider
        job.query = query if len(query) > 0 else None
        job.scrap_old_news = scrap_old_news
        if interval:
            job.interval = interval
            job.count = count
            if not job.schedule_job:
                job.schedule_job = make_periodic_task(job.id, schedule(interval, count))
            else:
                job.schedule_job.crontab = update_schedule(schedule(interval, count))
                job.schedule_job.save()
        job.save()
        return HttpResponseRedirect(reverse('edit_job', kwargs={'id': request.POST.get('job')}))


def schedule(interval, every):
    DAY = 'DAY'
    WEEK = 'WEEK'
    MONTH = 'MONTH'
    NON = 'NON'
    scheduler = {
        'minute': 0,
        'hour': 0,
        'day_of_month': '*',
        'month_of_year': '*',
        'day_of_week': '*'
    }

    if interval == DAY:
        scheduler['day_of_month'] = '*/{}'.format(every) if every else '*'
    if interval == WEEK:
        if every > 4:
            every = 4
        if every < 1:
            every = 1
        scheduler['day_of_month'] = '*/{}'.format(every * 7) if every else '*'
    if interval == MONTH:
        if every > 12:
            every = 12
        if every < 1:
            every = 1
        scheduler['day_of_month'] = 0
        scheduler['month_of_year'] = '*/{}'.format(every) if every else '*'
    if interval == NON:
        scheduler['minute'] = '*/2'
        scheduler['hour'] = '*'
    return scheduler


def validate_query(query):
    query = query.lower()
    block = [';', 'select', 'alter', 'drop', 'truncate', 'update']
    if any(ext in query for ext in block):
        print('Query is blocked')
        return BAD_QUERY
    else:
        return query
